/*
 * Copyright (c) 2001 Mark Fullmer and The Ohio State University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: flow-print.c,v 1.8 2001/02/10 04:40:12 maf Exp $
 */

/* file on command line will contain:

   file1  net1 net2 net3...
   file2  net1 net2 net3...
 */



#include <sys/time.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <time.h>
#include <fcntl.h>
#include <string.h>

#include "ftlib.h"
#include "sym.h"
#include "fmt.h"

#include "patricia.h"


#define FTIO_OUT_MAX 128

int main(argc, argv)
int argc;
char **argv;
{
  extern char *optarg;
  struct ftio ftio;

  FILE *desc_file;
  struct ftio ftio_out[FTIO_OUT_MAX];
  struct fts2rec_v5 *rec;
  char buf[161];
  int ftio_out_count, i;
  patricia_tree_t *pt;
  patricia_node_t *pn;
  prefix_t *pref;
  struct ftver ftv;

  if (argc != 2) {
    fputs ("must specify description filename\n", stderr);
    return 1;
  }

  if ((desc_file = fopen(argv[1], "r")) == NULL) {
    fprintf (stderr, "could not open description filename %s\n", argv[1]);
    return 1;
  }


  pt = New_Patricia(32);
  ftio_out_count = 0;

  while (fgets (buf, 160, desc_file) != NULL) {
    struct ftset ftset;
    char *s, *s2;
    int fd;

    /* Format: filename prefix1 prefix2 prefix3... */

    s = strchr(buf, '\n');
    *s = 0;

    s = strchr(buf, ' ');
    if (s == NULL) 
      break;


    /* Null terminate the filename and move to the first prefix */
    *s++ = 0;

    fd = open(buf, O_WRONLY | O_APPEND | O_CREAT, 0644);

    if (fd < 0) {
      fprintf (stderr, "cannot open for writing: %s\n", buf);
      return 1;
    }

    ftset_init(&ftset, 0);
    memset(&ftv, 0, sizeof(ftv));
    ftv.s_version = FT_IO_SVERSION;
    ftv.d_version = 5;
    ftv.agg_method = 1;
    ftv.agg_version = 2;



    if (ftio_init(ftio_out + ftio_out_count, fd, FT_IO_FLAG_WRITE) < 0) {
      fprintf (stderr, "cannot initialize for writing: %s\n", buf);
      return 1;
    }

    if (ftio_set_ver(ftio_out + ftio_out_count, &ftv) < 0) {
      fprintf (stderr, "cannot set version: %s\n", buf);
      return 1;
    }

    ftio_set_byte_order(ftio_out + ftio_out_count, ftset.byte_order);
    ftio_set_streaming(ftio_out + ftio_out_count, 0);

    if (ftio_write_header(ftio_out + ftio_out_count) < 0) {
      fprintf (stderr, "cannot write header: %s\n", buf);
      return 1;
    }


    while (s != NULL) {
      s2 = strchr(s, ' ');
      if (s2 != NULL) {
	*s2 = 0;
      }
      
      if ((pref = ascii2prefix(AF_INET, s)) == NULL) {
	fprintf (stderr, "invalid prefix: %s\n", s);
	return 1;
      }

      if ((pn = patricia_lookup(pt, pref)) == NULL) {
	fprintf (stderr, "could not create node for prefix: %s\n", s);
      }

      /* All we really want to store */
      pn->data = ftio_out + ftio_out_count;
      s = s2;
    }
    
    if (++ftio_out_count == FTIO_OUT_MAX) {
      fputs ("too many files, ignoring the rest.\n", stderr);
      break;
    }
  }
  fclose (desc_file);


  /* At this point we have the tree */

  /* read from stdin */
  if (ftio_init(&ftio, 0, FT_IO_FLAG_READ) < 0) {
    fprintf(stderr, "ftio_init(): failed\n");
    exit (1);
  }

  while ((rec = ftio_read(&ftio))) {
    prefix_t pref_tmp;

    /* FIXME - must be able to change this to srcaddr later */
    pref_tmp.add.sin.s_addr = ntohl(rec->dstaddr);
    
    if ((pref_tmp.bitlen = rec->dst_mask) == 0) {
      pref_tmp.bitlen = 32;
    }

    if ((pn = patricia_search_best(pt, &pref_tmp)) != NULL) {
      ftio_write((struct ftio *)pn->data, rec);
    }
  }


  /* We are done, close down. */
  for (i = 0; i < ftio_out_count; i++) {
    ftio_close(ftio_out + i);
  }

  return 0;

} /* main */

/* debug */



