/*
 * Copyright (c) 2001 Mark Fullmer and The Ohio State University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: flow-profile.h,v 1.5 2001/02/20 04:19:18 maf Exp $
 */

#if HAVE_CONFIG_H
 #include <config.h>
#endif

#include <sys/types.h>
#include "bytes.h"

#ifndef _FLOW_PROFILE_H
#define _FLOW_PROFILE_H

#define OPT_PERCENT 1   /* use percent's */
#define OPT_NAMES   2   /* use symbolic names */
#define OPT_LONG    4   /* long output */

/* save memory */
#define PROF_UDP_TCP_COMBINED 

#define DIR_OUT 0
#define DIR_IN 1

#ifdef PROF_UDP_TCP_COMBINED
#define udp_port_packets tcp_port_packets
#define udp_port_octets tcp_port_octets
#endif /* PROF_UDP_TCP_COMBINED */

struct string_list {
  size_t entries;
  char **ptr;
};

#define PROF_NAME_LEN 64+1

struct profile {

#ifndef PROF_UDP_TCP_COMBINED
  /* udp port counters */
  u_int64 udp_port_packets[2][1024];
  u_int64 udp_port_octets[2][1024];
#endif

  /* tcp port counters */
  u_int64 tcp_port_packets[2][1024];
  u_int64 tcp_port_octets[2][1024];

  /* tcp flag counters */
  u_int64 tcp_flag_syn[2];
  u_int64 tcp_flag_syn_ack[2];

  /* ip protocol counters */
  u_int64 ip_prot_octets[2][256];
  u_int64 ip_prot_packets[2][256];

}; /* 81952 (73760) bytes */

int profile_dump(struct profile *prof, int options, int tally, char *name,
  u_int64 *noctets, u_int64 *nflows, u_int64 *npackets, u_int64 *t_octets,
  u_int64 *t_packets);

/*
01234567890123456789012345678901234567890123456789012345678901234567890123456789
TTTTTTTTTTTTTTTTTTTTTTTTTTTTTT  NNNNNNNNNN  NNNNNNNNNN   NNNNNNNNN  NNNNNNNNNN
 NNNNNNNNNN NNNNNNNNNN NNNNNNNNNN NNNNNNNNNN   NNNNNNNNNN NNNNNNNNNN NNNNNNNNNN
*/
#endif

