/*
 * Copyright (c) 2001 Mark Fullmer and The Ohio State University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: hash_ip_s.h,v 1.5 2001/02/20 04:19:19 maf Exp $
 */

#if HAVE_CONFIG_H
 #include <config.h>
#endif

#include <sys/types.h>
#include "bytes.h"

int hash_ip_sinit();
int hash_ip_sdump();

#define HASH_IP_S_STAT 

struct hash_ip_s_addr {

  u_short seg;    /* segment pointer */
  u_short off;    /* offset */
#ifdef HASH_IP_S_STAT
  u_long  hits;
#endif /* HASH_IP_S_STAT */
};

struct hash_ip_s_rec {

  u_int32 addr;       /* ip address */
  int32 index;        /* index into the name table */

  struct hash_ip_s_addr next; /* next record */
};


struct hash_ip_s_table {
  struct hash_ip_s_addr head;   /* head pointer to free records */
  struct hash_ip_s_addr *table; /* lookup table */
  char **data;                  /* hash data */
  u_short lastoff;              /* last offset before need new allocation */
};

  
/* add a record to the table, or update the current record */
int hash_ip_s_update();

/* lookup next key */
int hash_ip_s_next();

/* display histogram if hash table */
int hash_ip_s_hist();

int hash_ip_s_init(struct hash_ip_s_table *ht);
int hash_ip_s_free(struct hash_ip_s_table *ht);
int hash_ip_s_update(struct hash_ip_s_table *ht, struct hash_ip_s_rec *newrec);
int hash_ip_s_lookup(struct hash_ip_s_table *ht, struct hash_ip_s_rec *newrec);
void hash_ip_stat_dump(struct hash_ip_s_table *ht);
void hash_ip_s_dump (struct hash_ip_s_table *ht);

